/*****************************/
/***** File: timing.h    *****/
/***** Timing functions! *****/
/*****************************/

#include <unistd.h>
#define ECKBOX_TIMING_H

/***** Global variables! *****/
unsigned long long int timing_constant;

/***** Function: rdtsc ()                      *****/
/***** Returns num of CPU cycles since startup *****/

unsigned long long int rdtsc ()
{
  unsigned long long int x;
  asm volatile (".byte 0x0f, 0x31" : "=A" (x));
  return x;
}

/***** Function: timing_cpu_clockspeed *****/
/***** Gets CPU clockspeed             *****/

unsigned long long int timing_cpu_clockspeed ()
{
  unsigned long long int t1, t2;
  t1=rdtsc ();
  sleep (1);
  t2=rdtsc ();
  return t2-t1;
}

/***** Function: timing_sleep (int)   *****/
/***** Sleeps x number of CPU cycles  *****/
/***** See timing_init at the bottom! *****/

void timing_sleep (int x)
{
  int i;
  if(timing_constant)
  {
    for (i=0; i<x/timing_constant; i++)
      asm ("nop");
  }
  else
  { 
    for (i=0; i<x; i++)
      asm ("nop");
  }
}

/***** Function: timing_get_pix *****/
/***** Returns pixel delay time *****/

unsigned long long int timing_get_pix ()
{
  return (timing_cpu_clockspeed()/(800*600*60));
}

/***** Function: timing_get_href  *****/
/***** Returns horizontal refresh *****/

unsigned long long int timing_get_href ()
{
  return (timing_cpu_clockspeed()/(600*60));
}

/***** Function: timing_get_vref *****/
/***** Returns vertical refresh  *****/

unsigned long long int timing_get_vref ()
{
  return (timing_cpu_clockspeed()/60);
}

/***** Function: timing_init                *****/
/***** Gets timing_constant, the num of cpu *****/ 
/***** cycles wasted in timing_sleep        *****/

int timing_init ()
{
  unsigned long long int t1, t2;
  t1=rdtsc();
  timing_sleep(2);
  t2=rdtsc();
  t1+=2; /* add two to t1, to account for sleep(2) */
  t2-=t1; /* gets total wasted cycles */
  t2/=2; /* divide by two, because we slept twice! */
  timing_constant=t2;
  return 0;
}
